@@ -8,9 +8,12 @@
 #include "dir.h"
 #include "cache-tree.h"
 #include "path-list.h"
+#include "parse-options.h"
 
-static const char builtin_mv_usage[] =
-"git-mv [-n] [-f] (<source> <destination> | [-k] <source>... <destination>)";
+static const char * const builtin_mv_usage[] = {
+	"git-mv [options] <source>... <destination>",
+	NULL
+};
 
 static const char **copy_pathspec(const char *prefix, const char **pathspec,
 				  int count, int base_name)
@@ -22,10 +25,7 @@ static const char **copy_pathspec(const char *prefix, const char **pathspec,
 	for (i = 0; i < count; i++) {
 		int length = strlen(result[i]);
 		if (length > 0 && result[i][length - 1] == '/') {
-			char *without_slash = xmalloc(length);
-			memcpy(without_slash, result[i], length - 1);
-			without_slash[length - 1] = '\0';
-			result[i] = without_slash;
+			result[i] = xmemdupz(result[i], length - 1);
 		}
 		if (base_name) {
 			const char *last_slash = strrchr(result[i], '/');
@@ -64,8 +64,14 @@ static struct lock_file lock_file;
 
 int cmd_mv(int argc, const char **argv, const char *prefix)
 {
-	int i, newfd, count;
+	int i, newfd;
 	int verbose = 0, show_only = 0, force = 0, ignore_errors = 0;
+	struct option builtin_mv_options[] = {
+		OPT__DRY_RUN(&show_only),
+		OPT_BOOLEAN('f', NULL, &force, "force move/rename even if target exists"),
+		OPT_BOOLEAN('k', NULL, &ignore_errors, "skip move/rename errors"),
+		OPT_END(),
+	};
 	const char **source, **destination, **dest_path;
 	enum update_mode { BOTH = 0, WORKING_DIRECTORY, INDEX } *modes;
 	struct stat st;
@@ -81,52 +87,29 @@ int cmd_mv(int argc, const char **argv, const char *prefix)
 	if (read_cache() < 0)
 		die("index file corrupt");
 
-	for (i = 1; i < argc; i++) {
-		const char *arg = argv[i];
+	argc = parse_options(argc, argv, builtin_mv_options, builtin_mv_usage, 0);
+	if (--argc < 1)
+		usage_with_options(builtin_mv_usage, builtin_mv_options);
 
-		if (arg[0] != '-')
-			break;
-		if (!strcmp(arg, "--")) {
-			i++;
-			break;
-		}
-		if (!strcmp(arg, "-n")) {
-			show_only = 1;
-			continue;
-		}
-		if (!strcmp(arg, "-f")) {
-			force = 1;
-			continue;
-		}
-		if (!strcmp(arg, "-k")) {
-			ignore_errors = 1;
-			continue;
-		}
-		usage(builtin_mv_usage);
-	}
-	count = argc - i - 1;
-	if (count < 1)
-		usage(builtin_mv_usage);
-
-	source = copy_pathspec(prefix, argv + i, count, 0);
-	modes = xcalloc(count, sizeof(enum update_mode));
-	dest_path = copy_pathspec(prefix, argv + argc - 1, 1, 0);
+	source = copy_pathspec(prefix, argv, argc, 0);
+	modes = xcalloc(argc, sizeof(enum update_mode));
+	dest_path = copy_pathspec(prefix, argv + argc, 1, 0);
 
 	if (dest_path[0][0] == '\0')
 		/* special case: "." was normalized to "" */
-		destination = copy_pathspec(dest_path[0], argv + i, count, 1);
+		destination = copy_pathspec(dest_path[0], argv, argc, 1);
 	else if (!lstat(dest_path[0], &st) &&
 			S_ISDIR(st.st_mode)) {
 		dest_path[0] = add_slash(dest_path[0]);
-		destination = copy_pathspec(dest_path[0], argv + i, count, 1);
+		destination = copy_pathspec(dest_path[0], argv, argc, 1);
 	} else {
-		if (count != 1)
-			usage(builtin_mv_usage);
+		if (argc != 1)
+			usage_with_options(builtin_mv_usage, builtin_mv_options);
 		destination = dest_path;
 	}
 
 	/* Checking */
-	for (i = 0; i < count; i++) {
+	for (i = 0; i < argc; i++) {
 		const char *src = source[i], *dst = destination[i];
 		int length, src_is_dir;
 		const char *bad = NULL;
@@ -170,29 +153,29 @@ int cmd_mv(int argc, const char **argv, const char *prefix)
 
 				if (last - first > 0) {
 					source = xrealloc(source,
-							(count + last - first)
+							(argc + last - first)
 							* sizeof(char *));
 					destination = xrealloc(destination,
-							(count + last - first)
+							(argc + last - first)
 							* sizeof(char *));
 					modes = xrealloc(modes,
-							(count + last - first)
+							(argc + last - first)
 							* sizeof(enum update_mode));
 				}
 
 				dst = add_slash(dst);
-				dst_len = strlen(dst) - 1;
+				dst_len = strlen(dst);
 
 				for (j = 0; j < last - first; j++) {
 					const char *path =
 						active_cache[first + j]->name;
-					source[count + j] = path;
-					destination[count + j] =
+					source[argc + j] = path;
+					destination[argc + j] =
 						prefix_path(dst, dst_len,
-							path + length);
-					modes[count + j] = INDEX;
+							path + length + 1);
+					modes[argc + j] = INDEX;
 				}
-				count += last - first;
+				argc += last - first;
 			}
 		} else if (lstat(dst, &st) == 0) {
 			bad = "destination exists";
@@ -219,12 +202,12 @@ int cmd_mv(int argc, const char **argv, const char *prefix)
 
 		if (bad) {
 			if (ignore_errors) {
-				if (--count > 0) {
+				if (--argc > 0) {
 					memmove(source + i, source + i + 1,
-						(count - i) * sizeof(char *));
+						(argc - i) * sizeof(char *));
 					memmove(destination + i,
 						destination + i + 1,
-						(count - i) * sizeof(char *));
+						(argc - i) * sizeof(char *));
 				}
 			} else
 				die ("%s, source=%s, destination=%s",
@@ -232,7 +215,7 @@ int cmd_mv(int argc, const char **argv, const char *prefix)
 		}
 	}
 
-	for (i = 0; i < count; i++) {
+	for (i = 0; i < argc; i++) {
 		const char *src = source[i], *dst = destination[i];
 		enum update_mode mode = modes[i];
 		if (show_only || verbose)
@@ -256,7 +239,7 @@ int cmd_mv(int argc, const char **argv, const char *prefix)
 			path_list_insert(dst, &added);
 	}
 
-        if (show_only) {
+	if (show_only) {
 		show_list("Changed  : ", &changed);
 		show_list("Adding   : ", &added);
 		show_list("Deleting : ", &deleted);
@@ -276,15 +259,11 @@ int cmd_mv(int argc, const char **argv, const char *prefix)
 			add_file_to_cache(path, verbose);
 		}
 
-		for (i = 0; i < deleted.nr; i++) {
-			const char *path = deleted.items[i].path;
-			remove_file_from_cache(path);
-			cache_tree_invalidate_path(active_cache_tree, path);
-		}
+		for (i = 0; i < deleted.nr; i++)
+			remove_file_from_cache(deleted.items[i].path);
 
 		if (active_cache_changed) {
 			if (write_cache(newfd, active_cache, active_nr) ||
-			    close(newfd) ||
 			    commit_locked_index(&lock_file))
 				die("Unable to write new index file");
 		}
