@@ -11,7 +11,7 @@ There is the head (called H) and another commit (called M), which is
 simply ahead of H.  The index and the work tree contains a state that
 is derived from H, but may also have local changes.  This test checks
 all the combinations described in the two-tree merge "carry forward"
-rules, found in <Documentation/git-read-tree.txt>.
+rules, found in <Documentation/git read-tree.txt>.
 
 In the test, these paths are used:
         bozbar  - in H, stays in M, modified from bozbar to gnusto
@@ -23,7 +23,7 @@ In the test, these paths are used:
 . ./test-lib.sh
 
 read_tree_twoway () {
-    git-read-tree -m "$1" "$2" && git-ls-files --stage
+    git read-tree -m "$1" "$2" && git ls-files --stage
 }
 
 _x40='[0-9a-f][0-9a-f][0-9a-f][0-9a-f][0-9a-f]'
@@ -37,7 +37,7 @@ compare_change () {
 }
 
 check_cache_at () {
-	clean_if_empty=`git-diff-files -- "$1"`
+	clean_if_empty=`git diff-files -- "$1"`
 	case "$clean_if_empty" in
 	'')  echo "$1: clean" ;;
 	?*)  echo "$1: dirty" ;;
@@ -68,24 +68,24 @@ test_expect_success \
      cat bozbar-old >bozbar &&
      echo rezrov >rezrov &&
      echo yomin >yomin &&
-     git-update-index --add nitfol bozbar rezrov &&
-     treeH=`git-write-tree` &&
+     git update-index --add nitfol bozbar rezrov &&
+     treeH=`git write-tree` &&
      echo treeH $treeH &&
-     git-ls-tree $treeH &&
+     git ls-tree $treeH &&
 
      cat bozbar-new >bozbar &&
-     git-update-index --add frotz bozbar --force-remove rezrov &&
-     git-ls-files --stage >M.out &&
-     treeM=`git-write-tree` &&
+     git update-index --add frotz bozbar --force-remove rezrov &&
+     git ls-files --stage >M.out &&
+     treeM=`git write-tree` &&
      echo treeM $treeM &&
-     git-ls-tree $treeM &&
-     git-diff-tree $treeH $treeM'
+     git ls-tree $treeM &&
+     git diff-tree $treeH $treeM'
 
 test_expect_success \
     '1, 2, 3 - no carry forward' \
     'rm -f .git/index &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >1-3.out &&
+     git ls-files --stage >1-3.out &&
      git diff M.out 1-3.out &&
      check_cache_at bozbar dirty &&
      check_cache_at frotz dirty &&
@@ -96,11 +96,11 @@ echo '+100644 X 0	yomin' >expected
 test_expect_success \
     '4 - carry forward local addition.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
-     git-update-index --add yomin &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
+     git update-index --add yomin &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >4.out || return 1
+     git ls-files --stage >4.out || return 1
      git diff M.out 4.out >4diff.out
      compare_change 4diff.out expected &&
      check_cache_at yomin clean'
@@ -108,13 +108,13 @@ test_expect_success \
 test_expect_success \
     '5 - carry forward local addition.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo yomin >yomin &&
-     git-update-index --add yomin &&
+     git update-index --add yomin &&
      echo yomin yomin >yomin &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >5.out || return 1
+     git ls-files --stage >5.out || return 1
      git diff M.out 5.out >5diff.out
      compare_change 5diff.out expected &&
      check_cache_at yomin dirty'
@@ -122,83 +122,83 @@ test_expect_success \
 test_expect_success \
     '6 - local addition already has the same.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
-     git-update-index --add frotz &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
+     git update-index --add frotz &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >6.out &&
+     git ls-files --stage >6.out &&
      git diff M.out 6.out &&
      check_cache_at frotz clean'
 
 test_expect_success \
     '7 - local addition already has the same.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo frotz >frotz &&
-     git-update-index --add frotz &&
+     git update-index --add frotz &&
      echo frotz frotz >frotz &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >7.out &&
+     git ls-files --stage >7.out &&
      git diff M.out 7.out &&
      check_cache_at frotz dirty'
 
 test_expect_success \
     '8 - conflicting addition.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo frotz frotz >frotz &&
-     git-update-index --add frotz &&
+     git update-index --add frotz &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
 test_expect_success \
     '9 - conflicting addition.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo frotz frotz >frotz &&
-     git-update-index --add frotz &&
+     git update-index --add frotz &&
      echo frotz >frotz &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
 test_expect_success \
     '10 - path removed.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo rezrov >rezrov &&
-     git-update-index --add rezrov &&
+     git update-index --add rezrov &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >10.out &&
+     git ls-files --stage >10.out &&
      git diff M.out 10.out'
 
 test_expect_success \
     '11 - dirty path removed.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo rezrov >rezrov &&
-     git-update-index --add rezrov &&
+     git update-index --add rezrov &&
      echo rezrov rezrov >rezrov &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
 test_expect_success \
     '12 - unmatching local changes being removed.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo rezrov rezrov >rezrov &&
-     git-update-index --add rezrov &&
+     git update-index --add rezrov &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
 test_expect_success \
     '13 - unmatching local changes being removed.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo rezrov rezrov >rezrov &&
-     git-update-index --add rezrov &&
+     git update-index --add rezrov &&
      echo rezrov >rezrov &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
@@ -210,12 +210,12 @@ EOF
 test_expect_success \
     '14 - unchanged in two heads.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo nitfol nitfol >nitfol &&
-     git-update-index --add nitfol &&
+     git update-index --add nitfol &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >14.out || return 1
+     git ls-files --stage >14.out || return 1
      git diff M.out 14.out >14diff.out
      compare_change 14diff.out expected &&
      check_cache_at nitfol clean'
@@ -223,13 +223,13 @@ test_expect_success \
 test_expect_success \
     '15 - unchanged in two heads.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo nitfol nitfol >nitfol &&
-     git-update-index --add nitfol &&
+     git update-index --add nitfol &&
      echo nitfol nitfol nitfol >nitfol &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >15.out || return 1
+     git ls-files --stage >15.out || return 1
      git diff M.out 15.out >15diff.out
      compare_change 15diff.out expected &&
      check_cache_at nitfol dirty'
@@ -237,66 +237,66 @@ test_expect_success \
 test_expect_success \
     '16 - conflicting local change.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo bozbar bozbar >bozbar &&
-     git-update-index --add bozbar &&
+     git update-index --add bozbar &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
 test_expect_success \
     '17 - conflicting local change.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      echo bozbar bozbar >bozbar &&
-     git-update-index --add bozbar &&
+     git update-index --add bozbar &&
      echo bozbar bozbar bozbar >bozbar &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
 test_expect_success \
     '18 - local change already having a good result.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      cat bozbar-new >bozbar &&
-     git-update-index --add bozbar &&
+     git update-index --add bozbar &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >18.out &&
+     git ls-files --stage >18.out &&
      git diff M.out 18.out &&
      check_cache_at bozbar clean'
 
 test_expect_success \
     '19 - local change already having a good result, further modified.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      cat bozbar-new >bozbar &&
-     git-update-index --add bozbar &&
+     git update-index --add bozbar &&
      echo gnusto gnusto >bozbar &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >19.out &&
+     git ls-files --stage >19.out &&
      git diff M.out 19.out &&
      check_cache_at bozbar dirty'
 
 test_expect_success \
     '20 - no local change, use new tree.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      cat bozbar-old >bozbar &&
-     git-update-index --add bozbar &&
+     git update-index --add bozbar &&
      read_tree_twoway $treeH $treeM &&
-     git-ls-files --stage >20.out &&
+     git ls-files --stage >20.out &&
      git diff M.out 20.out &&
      check_cache_at bozbar dirty'
 
 test_expect_success \
     '21 - no local change, dirty cache.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      cat bozbar-old >bozbar &&
-     git-update-index --add bozbar &&
+     git update-index --add bozbar &&
      echo gnusto gnusto >bozbar &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
@@ -304,10 +304,10 @@ test_expect_success \
 test_expect_success \
     '22 - local change cache updated.' \
     'rm -f .git/index &&
-     git-read-tree $treeH &&
-     git-checkout-index -u -f -q -a &&
+     git read-tree $treeH &&
+     git checkout-index -u -f -q -a &&
      sed -e "s/such as/SUCH AS/" bozbar-old >bozbar &&
-     git-update-index --add bozbar &&
+     git update-index --add bozbar &&
      if read_tree_twoway $treeH $treeM; then false; else :; fi'
 
 # Also make sure we did not break DF vs DF/DF case.
@@ -315,28 +315,28 @@ test_expect_success \
     'DF vs DF/DF case setup.' \
     'rm -f .git/index &&
      echo DF >DF &&
-     git-update-index --add DF &&
-     treeDF=`git-write-tree` &&
+     git update-index --add DF &&
+     treeDF=`git write-tree` &&
      echo treeDF $treeDF &&
-     git-ls-tree $treeDF &&
+     git ls-tree $treeDF &&
 
      rm -f DF &&
      mkdir DF &&
      echo DF/DF >DF/DF &&
-     git-update-index --add --remove DF DF/DF &&
-     treeDFDF=`git-write-tree` &&
+     git update-index --add --remove DF DF/DF &&
+     treeDFDF=`git write-tree` &&
      echo treeDFDF $treeDFDF &&
-     git-ls-tree $treeDFDF &&
-     git-ls-files --stage >DFDF.out'
+     git ls-tree $treeDFDF &&
+     git ls-files --stage >DFDF.out'
 
 test_expect_success \
     'DF vs DF/DF case test.' \
     'rm -f .git/index &&
      rm -fr DF &&
      echo DF >DF &&
-     git-update-index --add DF &&
+     git update-index --add DF &&
      read_tree_twoway $treeDF $treeDFDF &&
-     git-ls-files --stage >DFDFcheck.out &&
+     git ls-files --stage >DFDFcheck.out &&
      git diff DFDF.out DFDFcheck.out &&
      check_cache_at DF/DF dirty &&
      :'
