@@ -3,7 +3,7 @@
 # Copyright (c) 2006 Junio C Hamano
 #
 
-test_description='git-apply with new style GNU diff with empty context
+test_description='git apply with new style GNU diff with empty context
 
 '
 
@@ -53,4 +53,3 @@ test_expect_success 'apply --apply' '
 '
 
 test_done
-
