@@ -3,20 +3,20 @@
 # Copyright (c) 2005 Junio C Hamano
 #
 
-test_description='git-mailinfo and git-mailsplit test'
+test_description='git mailinfo and git mailsplit test'
 
 . ./test-lib.sh
 
 test_expect_success 'split sample box' \
-	'git-mailsplit -o. ../t5100/sample.mbox >last &&
+	'git mailsplit -o. ../t5100/sample.mbox >last &&
 	last=`cat last` &&
 	echo total is $last &&
-	test `cat last` = 8'
+	test `cat last` = 9'
 
 for mail in `echo 00*`
 do
 	test_expect_success "mailinfo $mail" \
-		"git-mailinfo -u msg$mail patch$mail <$mail >info$mail &&
+		"git mailinfo -u msg$mail patch$mail <$mail >info$mail &&
 		echo msg &&
 		diff ../t5100/msg$mail msg$mail &&
 		echo patch &&
